from django.urls import resolve
from django.test import TestCase
from django.http import HttpRequest
from django.template.loader import render_to_string

from lists.views import home_page
from lists.models import Item

class HomePageTest(TestCase):

    def test_root_url_resolves_to_home_page_view(self):
        found = resolve('/')
        self.assertEqual(found.func, home_page)

    def test_uses_home_template(self):
        response = self.client.get('/')

        # The Manual Way of doing it:
        html = response.content.decode('utf8')  
        self.assertTrue(html.startswith('<html>'))
        self.assertIn('<title>To-Do lists</title>', html)
        self.assertTrue(html.strip().endswith('</html>'))

        # Using Django's Builtin Testing Client:
        self.assertTemplateUsed(response, 'home.html')

    def test_can_save_POST_request(self):
        response = self.client.post('/', data={'item_text': 'A new list item'})
        self.assertIn('A new list item', response.content.decode())
        self.assertTemplateUsed(response, 'home.html')

class ItemModelTest(TestCase):

    def test_save_and_retrieve_items(self):
        item1 = Item()
        item1.text = 'The first (ever) list item'
        item1.save()
        
        item2 = Item()
        item2.text = 'Item the 2nd'
        item2.save()

        saved = Item.objects.all()
        self.assertEqual(saved.count(), 2)

        first_item = saved[0]
        second_item = saved[1]
        self.assertEqual(first_item.text, item1.text)
        self.assertEqual(second_item.text, item2.text)
